@@ -10,10 +10,7 @@
 #define PTS_EVERYTHING 1e18
 #define VIDEO_TIME_BASE 1000
 #include <Rinternals.h>
-
-#if LIBAVCODEC_VERSION_MAJOR >= 59
-#define NEW_CHANNEL_API
-#endif
+#include "avcompat.h"
 
 int total_open_handles = 0;
 
@@ -670,6 +667,6 @@ SEXP R_convert_audio(SEXP audio, SEXP out_file, SEXP out_format, SEXP out_channe
   return out_file;
 }
 
-SEXP R_get_open_handles(){
+SEXP R_get_open_handles(void){
   return Rf_ScalarInteger(total_open_handles);
 }
